import cguard

import pytest

from cguard.core.approval import Approval
from cguard.requestor import GuardRequestor
from cguard.test.util import mock_response

approvial_json = {
    "id": "guard_session_123",
    "state": "approved",
    "requester": {"id": "user_123", "email": "user@example.com"},
    "guard_application": {
        "id": "guard_application_abc",
        "name": "abc",
        "settings": {"message_of_the_day": None, "reason_required": True},
    },
}

denial_json = {
    "id": "guard_session_123",
    "state": "denied",
    "requester": {"id": "user_123", "email": "user@example.com"},
    "guard_application": {
        "id": "guard_application_abc",
        "name": "abc",
        "settings": {"message_of_the_day": None, "reason_required": True},
    },
}


class TestApproval(object):
    def test_wait_for_approval_approved(self):
        with mock_response(json_data=approvial_json):
            approval = Approval()
            assert (
                approval.wait_for_approval(
                    "abc",
                    "token_123",
                    "guard_session_123",
                    "user_123",
                    "Waiting for approval..",
                )
                == True
            )


class TestApproval(object):
    def test_wait_for_approval_denied(self):
        with mock_response(json_data=denial_json):
            approval = Approval()
            with pytest.raises(SystemExit):
                # Assert the program exits on denied
                approval.wait_for_approval(
                    "abc",
                    "token_123",
                    "guard_session_123",
                    "user_123",
                    "Waiting for approval..",
                )



import pytest
from cguard.test.util import mock_requests
from cguard.requestor import HTTPClient

import requests

import responses
from responses import GET, POST, PATCH, PUT, DELETE, HEAD


class TestHTTPClient(object):
    @responses.activate
    def test_http_client_get(self):
        mock_requests("/sessions", method=GET)

        assert (
            HTTPClient.make_request(
                "get",
                "https://example.com/sessions",
            ).status_code
            == 200
        )

    @responses.activate
    def test_http_client_post(self):
        mock_requests("/sessions", method=POST)

        assert (
            HTTPClient.make_request(
                "post", "https://example.com/sessions", data={"foo": "bar"}
            ).status_code
            == 200
        )

    @responses.activate
    def test_http_client_put(self):
        mock_requests("/sessions", method=PUT)

        assert (
            HTTPClient.make_request(
                "put", "https://example.com/sessions", data={"foo": "bar"}
            ).status_code
            == 200
        )

    @responses.activate
    def test_http_client_patch(self):
        mock_requests("/sessions", method=PATCH)

        assert (
            HTTPClient.make_request(
                "patch", "https://example.com/sessions", data={"foo": "bar"}
            ).status_code
            == 200
        )

    @responses.activate
    def test_http_client_head(self):
        mock_requests("/sessions", method=HEAD)

        assert (
            HTTPClient.make_request(
                "head",
                "https://example.com/sessions",
            ).status_code
            == 200
        )

    @responses.activate
    def test_http_client_delete(self):
        mock_requests("/sessions", method=DELETE)

        assert (
            HTTPClient.make_request(
                "delete",
                "https://example.com/sessions",
            ).status_code
            == 200
        )



from unittest.mock import ANY
from unittest.mock import patch

import pytest

from os.path import expanduser

from cguard.util import home_dir, cguard_dir, shims_dir


class TestUtil(object):
    def test_home_dir(self):
        d = home_dir()
        assert d == expanduser("~")

    def test_cguard_dir(self):
        target = expanduser("~") + "/.cguard"
        d = cguard_dir()
        assert d == target

    def test_shims_dir(self):
        target = expanduser("~") + "/.cguard/shims"
        d = shims_dir()
        assert d == target



import pytest

import cguard
from cguard.requestor import GuardRequestor
from cguard.test.util import mock_response


app_name = "test-app"
app_token = "12345"
user_token = "abcdefg"
program_args = "run"


class TestAPIRequestor(object):
    def test_request_access(self):
        requestor = GuardRequestor()

        with mock_response():
            assert (
                requestor.request_access(
                    app_name,
                    app_token,
                    user_token,
                    program_args,
                    "hostname",
                ).status_code
                == 200
            )

    def test_request_access_is_called_correctly(self):
        requestor = GuardRequestor()

        with mock_response():
            assert requestor.request_access(
                app_name,
                app_token,
                user_token,
                program_args,
                "hostname",
            )

            cguard.requestor.HTTPClient.make_request.assert_called_with(
                "post",
                "https://api.cased.com/guard/sessions?user_token=abcdefg",
                data={
                    "guard_application_id": "test-app",
                    "metadata": {"hostname": "hostname"},
                    "command": "run",
                    "reason": None,
                },
                key="12345",
            )

    def test_request_access_is_called_correctly_with_reason(self):
        requestor = GuardRequestor()

        with mock_response():
            assert requestor.request_access(
                app_name,
                app_token,
                user_token,
                program_args,
                "hostname",
                "reason for access",
            )

            cguard.requestor.HTTPClient.make_request.assert_called_with(
                "post",
                "https://api.cased.com/guard/sessions?user_token=abcdefg",
                data={
                    "guard_application_id": "test-app",
                    "metadata": {"hostname": "hostname"},
                    "command": "run",
                    "reason": "reason for access",
                },
                key="12345",
            )

    def test_get_session(self):
        requestor = GuardRequestor()
        with mock_response():
            assert (
                requestor.get_session(
                    app_name, app_token, "session_1234", "user_12345"
                ).status_code
                == 200
            )

    def test_get_session_is_called_correctly(self):
        requestor = GuardRequestor()
        with mock_response():
            assert requestor.get_session(
                app_name, app_token, "session_1234", "user_12345"
            )

            cguard.requestor.HTTPClient.make_request.assert_called_with(
                "get",
                "https://api.cased.com/guard/sessions/session_1234?user_token=user_12345",
                data={
                    "guard_application_id": "test-app",
                },
                key="12345",
            )

    def test_cancel_session(self):
        requestor = GuardRequestor()
        with mock_response():
            assert (
                requestor.cancel_session(
                    app_token, "session_1234", "user_12345"
                ).status_code
                == 200
            )

    def test_cancel_session_is_called_correctly(self):
        requestor = GuardRequestor()
        with mock_response():
            assert requestor.cancel_session(app_token, "session_1234", "user_12345")

            cguard.requestor.HTTPClient.make_request.assert_called_with(
                "post",
                "https://api.cased.com/guard/sessions/session_1234/cancel?user_token=user_12345",
                key="12345",
            )

    def test_get_applications(self):
        requestor = GuardRequestor()
        with mock_response():
            assert requestor.get_applications(user_token, "local").status_code == 200

    def test_get_applications_is_called_correctly(self):
        requestor = GuardRequestor()
        with mock_response():
            assert requestor.get_applications(user_token, "local")

            cguard.requestor.HTTPClient.make_request.assert_called_with(
                "get",
                "https://api.cased.com/guard/applications?user_token=abcdefg&environment=local",
            )



from cguard.access_manager.access_manager import AccessManager
from cguard.requestor import GuardRequestor


class CasedGuardAccessManager(AccessManager):
    def __init__(self):
        self.requestor = GuardRequestor()

    def check_access(self, app_name, app_token, session_id, user_token):
        res = self.requestor.get_session(app_name, app_token, session_id, user_token)
        state = res.json().get("state")
        return state



from os.path import expanduser

import json


def home_dir():
    return expanduser("~")


def cguard_dir():
    return home_dir() + "/.cguard"


def shims_dir():
    return cguard_dir() + "/shims"


def read_settings():
    try:
        filepath = cguard_dir() + "/" + "settings"
        with open(filepath, "r") as json_file:
            settings = json.load(json_file)

        return settings
    except:
        return {}


def poll_interval():
    settings = read_settings()
    interval = settings.get("poll") or 1
    return interval


def log_level():
    settings = read_settings()
    level = settings.get("log_level") or "info"
    return level


def environment():
    settings = read_settings()
    level = settings.get("environment") or "local"
    return level


def autosync():
    settings = read_settings()
    sync = settings.get("autosync") or True
    return sync


def output(msg):
    print("[guard 🔒] " + msg)


def debug(msg):
    print("[guard debug] " + msg)